@@ -9,8 +9,11 @@ __all__ = [
     'http',
     'io',
     'mqtt',
+    's3'
 ]
 
+__version__ = '0.11.19'
+
 
 class NativeResource:
     """
