@@ -154,7 +154,7 @@ static bool s_s3_auto_ranged_put_update(
                 meta_request,
                 AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_CREATE_MULTIPART_UPLOAD,
                 0,
-                AWS_S3_REQUEST_DESC_RECORD_RESPONSE_HEADERS);
+                AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS);
 
             auto_ranged_put->synced_data.create_multipart_upload_sent = true;
 
@@ -187,7 +187,7 @@ static bool s_s3_auto_ranged_put_update(
 
             /* Allocate a request for another part. */
             request = aws_s3_request_new(
-                meta_request, AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_PART, 0, AWS_S3_REQUEST_DESC_RECORD_RESPONSE_HEADERS);
+                meta_request, AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_PART, 0, AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS);
 
             request->part_number = auto_ranged_put->threaded_update_data.next_part_number;
 
@@ -221,7 +221,7 @@ static bool s_s3_auto_ranged_put_update(
                 meta_request,
                 AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_COMPLETE_MULTIPART_UPLOAD,
                 0,
-                AWS_S3_REQUEST_DESC_RECORD_RESPONSE_HEADERS);
+                AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS);
 
             auto_ranged_put->synced_data.complete_multipart_upload_sent = true;
 
@@ -285,7 +285,7 @@ static bool s_s3_auto_ranged_put_update(
                 meta_request,
                 AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_ABORT_MULTIPART_UPLOAD,
                 0,
-                AWS_S3_REQUEST_DESC_RECORD_RESPONSE_HEADERS);
+                AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS | AWS_S3_REQUEST_FLAG_ALWAYS_SEND);
 
             auto_ranged_put->synced_data.abort_multipart_upload_sent = true;
 
@@ -368,7 +368,7 @@ static int s_s3_auto_ranged_put_prepare_request(
             }
 
             /* Create a new put-object message to upload a part. */
-            message = aws_s3_put_object_message_new(
+            message = aws_s3_upload_part_message_new(
                 meta_request->allocator,
                 meta_request->initial_request_message,
                 &request->request_body,
@@ -557,6 +557,10 @@ static void s_s3_auto_ranged_put_request_finished(
                 }
             }
 
+            aws_s3_meta_request_lock_synced_data(meta_request);
+
+            ++auto_ranged_put->synced_data.num_parts_completed;
+
             AWS_LOGF_DEBUG(
                 AWS_LS_S3_META_REQUEST,
                 "id=%p: %d out of %d parts have completed.",
@@ -564,10 +568,6 @@ static void s_s3_auto_ranged_put_request_finished(
                 auto_ranged_put->synced_data.num_parts_completed,
                 auto_ranged_put->synced_data.total_num_parts);
 
-            aws_s3_meta_request_lock_synced_data(meta_request);
-
-            ++auto_ranged_put->synced_data.num_parts_completed;
-
             if (error_code == AWS_ERROR_SUCCESS) {
                 AWS_ASSERT(etag != NULL);
 
