@@ -25,7 +25,7 @@ int s2n_stuffer_write_network_order(struct s2n_stuffer *stuffer, const uint64_t
 {
     POSIX_ENSURE(length <= sizeof(input), S2N_ERR_SAFETY);
     POSIX_GUARD(s2n_stuffer_skip_write(stuffer, length));
-    uint8_t *data = stuffer->blob.data + stuffer->write_cursor - length;
+    uint8_t *data = (stuffer->blob.data) ? (stuffer->blob.data + stuffer->write_cursor - length) : NULL;
 
     for (int i = 0; i < length; i++) {
         S2N_INVARIANT(i <= length);
