@@ -304,6 +304,7 @@ struct s2n_connection {
     s2n_session_ticket_status session_ticket_status;
     struct s2n_blob client_ticket;
     uint32_t ticket_lifetime_hint;
+    struct s2n_ticket_fields tls13_ticket_fields;
 
     /* Session ticket extension from client to attempt to decrypt as the server. */
     uint8_t ticket_ext_data[S2N_TLS12_TICKET_SIZE_IN_BYTES];
@@ -357,6 +358,7 @@ struct s2n_connection {
     s2n_early_data_state early_data_state;
     uint32_t server_max_early_data_size;
     struct s2n_blob server_early_data_context;
+    uint32_t server_keying_material_lifetime;
 };
 
 int s2n_connection_is_managed_corked(const struct s2n_connection *s2n_connection);
