@@ -366,11 +366,21 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             SERVER_HELLO
     },
 
+    [INITIAL | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            SERVER_HELLO
+    },
+
     [INITIAL | HELLO_RETRY_REQUEST] = {
             CLIENT_HELLO,
             HELLO_RETRY_MSG
     },
 
+    [INITIAL | HELLO_RETRY_REQUEST | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            HELLO_RETRY_MSG
+    },
+
     [NEGOTIATED] = {
             CLIENT_HELLO,
             SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_FINISHED,
@@ -392,7 +402,21 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_FINISHED,
+            CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | MIDDLEBOX_COMPAT | WITH_EARLY_DATA] = {
+            CLIENT_HELLO,
+            SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_FINISHED,
+            CLIENT_CHANGE_CIPHER_SPEC, END_OF_EARLY_DATA, CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
+    [NEGOTIATED | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS | WITH_EARLY_DATA] = {
             CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
             SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_FINISHED,
             END_OF_EARLY_DATA, CLIENT_FINISHED,
@@ -417,6 +441,15 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | HELLO_RETRY_REQUEST | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            HELLO_RETRY_MSG, SERVER_CHANGE_CIPHER_SPEC,
+            CLIENT_HELLO,
+            SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_FINISHED,
+            CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE] = {
             CLIENT_HELLO,
             SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
@@ -431,6 +464,13 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | FULL_HANDSHAKE | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST] = {
             CLIENT_HELLO,
             HELLO_RETRY_MSG,
@@ -449,6 +489,15 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            HELLO_RETRY_MSG, SERVER_CHANGE_CIPHER_SPEC,
+            CLIENT_HELLO,
+            SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST | CLIENT_AUTH] = {
             CLIENT_HELLO,
             HELLO_RETRY_MSG,
@@ -467,6 +516,15 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST | CLIENT_AUTH | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            HELLO_RETRY_MSG, SERVER_CHANGE_CIPHER_SPEC,
+            CLIENT_HELLO,
+            SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_CERT, CLIENT_CERT_VERIFY, CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST | CLIENT_AUTH | NO_CLIENT_CERT] = {
             CLIENT_HELLO,
             HELLO_RETRY_MSG,
@@ -485,6 +543,15 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | FULL_HANDSHAKE | HELLO_RETRY_REQUEST | CLIENT_AUTH | NO_CLIENT_CERT | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            HELLO_RETRY_MSG, SERVER_CHANGE_CIPHER_SPEC,
+            CLIENT_HELLO,
+            SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_CERT, CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE | CLIENT_AUTH] = {
             CLIENT_HELLO,
             SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
@@ -499,6 +566,13 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             APPLICATION_DATA
     },
 
+    [NEGOTIATED | FULL_HANDSHAKE | CLIENT_AUTH | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_CERT, CLIENT_CERT_VERIFY, CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
+
     [NEGOTIATED | FULL_HANDSHAKE | CLIENT_AUTH | NO_CLIENT_CERT] = {
             CLIENT_HELLO,
             SERVER_HELLO, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
@@ -512,10 +586,17 @@ static message_type_t tls13_handshakes[S2N_HANDSHAKES_COUNT][S2N_MAX_HANDSHAKE_L
             CLIENT_CHANGE_CIPHER_SPEC, CLIENT_CERT, CLIENT_FINISHED,
             APPLICATION_DATA
     },
+
+    [NEGOTIATED | FULL_HANDSHAKE | CLIENT_AUTH | NO_CLIENT_CERT | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS] = {
+            CLIENT_HELLO, CLIENT_CHANGE_CIPHER_SPEC,
+            SERVER_HELLO, SERVER_CHANGE_CIPHER_SPEC, ENCRYPTED_EXTENSIONS, SERVER_CERT_REQ, SERVER_CERT, SERVER_CERT_VERIFY, SERVER_FINISHED,
+            CLIENT_CERT, CLIENT_FINISHED,
+            APPLICATION_DATA
+    },
 };
 /* clang-format on */
 
-#define MAX_HANDSHAKE_TYPE_LEN 115
+#define MAX_HANDSHAKE_TYPE_LEN 123
 static char handshake_type_str[S2N_HANDSHAKES_COUNT][MAX_HANDSHAKE_TYPE_LEN] = {0};
 
 static const char* tls12_handshake_type_names[] = {
@@ -535,7 +616,8 @@ static const char* tls13_handshake_type_names[] = {
     "NO_CLIENT_CERT|",
     "HELLO_RETRY_REQUEST|",
     "MIDDLEBOX_COMPAT|",
-    "WITH_EARLY_DATA|"
+    "WITH_EARLY_DATA|",
+    "EARLY_CLIENT_CCS|",
 };
 
 #define IS_TLS13_HANDSHAKE( conn )    ((conn)->actual_protocol_version == S2N_TLS13)
@@ -662,11 +744,12 @@ bool s2n_is_hello_retry_handshake(struct s2n_connection *conn)
 static S2N_RESULT s2n_conn_set_tls13_handshake_type(struct s2n_connection *conn) {
     RESULT_ENSURE_REF(conn);
 
-    bool is_hello_retry = IS_HELLO_RETRY_HANDSHAKE(conn);
-    RESULT_GUARD(s2n_handshake_type_reset(conn));
-    if (is_hello_retry) {
-        RESULT_GUARD(s2n_handshake_type_set_tls13_flag(conn, HELLO_RETRY_REQUEST));
-    }
+    /* Most handshake type flags should be reset before we calculate the handshake type,
+     * in order to handle changes during retries.
+     * However, flags that have already affected the message order must be kept to avoid
+     * rewriting the past.
+     */
+    conn->handshake.handshake_type &= (HELLO_RETRY_REQUEST | MIDDLEBOX_COMPAT | EARLY_CLIENT_CCS);
 
     /* A handshake type has been negotiated */
     RESULT_GUARD(s2n_handshake_type_set_flag(conn, NEGOTIATED));
@@ -692,9 +775,7 @@ static S2N_RESULT s2n_conn_set_tls13_handshake_type(struct s2n_connection *conn)
         RESULT_GUARD(s2n_handshake_type_set_flag(conn, CLIENT_AUTH));
     }
 
-    /* Use middlebox compatibility mode for TLS1.3 by default.
-    * For now, only disable it when QUIC support is enabled. */
-    if (!conn->config->quic_enabled) {
+    if (s2n_is_middlebox_compat_enabled(conn)) {
         RESULT_GUARD(s2n_handshake_type_set_tls13_flag(conn, MIDDLEBOX_COMPAT));
     }
 
@@ -788,24 +869,6 @@ int s2n_conn_set_handshake_no_client_cert(struct s2n_connection *conn)
     return 0;
 }
 
-int s2n_conn_set_handshake_read_block(struct s2n_connection *conn)
-{
-    POSIX_ENSURE_REF(conn);
-
-    conn->handshake.paused = 1;
-
-    return 0;
-}
-
-int s2n_conn_clear_handshake_read_block(struct s2n_connection *conn)
-{
-    POSIX_ENSURE_REF(conn);
-
-    conn->handshake.paused = 0;
-
-    return 0;
-}
-
 const char *s2n_connection_get_last_message_name(struct s2n_connection *conn)
 {
     PTR_ENSURE_REF(conn);
@@ -1038,6 +1101,15 @@ static int s2n_try_delete_session_cache(struct s2n_connection *conn)
     return 0;
 }
 
+static S2N_RESULT s2n_wipe_record(struct s2n_connection *conn)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_GUARD_POSIX(s2n_stuffer_wipe(&conn->header_in));
+    RESULT_GUARD_POSIX(s2n_stuffer_wipe(&conn->in));
+    conn->in_status = ENCRYPTED;
+    return S2N_RESULT_OK;
+}
+
 /* Reading is a little more complicated than writing as the TLS RFCs allow content
  * types to be interleaved at the record layer. We may get an alert message
  * during the handshake phase, or messages of types that we don't support (e.g.
@@ -1057,7 +1129,32 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
         record_type = TLS_HANDSHAKE;
         POSIX_GUARD_RESULT(s2n_quic_read_handshake_message(conn, &message_type));
     } else {
-        POSIX_GUARD(s2n_read_full_record(conn, &record_type, &isSSLv2));
+        int r = s2n_read_full_record(conn, &record_type, &isSSLv2);
+
+        /**
+         *= https://tools.ietf.org/rfc/rfc8446#section-4.2.10
+         *# If the client attempts a 0-RTT handshake but the server
+         *# rejects it, the server will generally not have the 0-RTT record
+         *# protection keys and must instead use trial decryption (either with
+         *# the 1-RTT handshake keys or by looking for a cleartext ClientHello in
+         *# the case of a HelloRetryRequest) to find the first non-0-RTT message.
+         *#
+         *# If the server chooses to accept the "early_data" extension, then it
+         *# MUST comply with the same error-handling requirements specified for
+         *# all records when processing early data records.  Specifically, if the
+         *# server fails to decrypt a 0-RTT record following an accepted
+         *# "early_data" extension, it MUST terminate the connection with a
+         *# "bad_record_mac" alert as per Section 5.2.
+         */
+        if ((r < 0) && (s2n_errno == S2N_ERR_DECRYPT) /* Decryption Error */
+                && (conn->mode == S2N_SERVER) /* On the server */
+                && (conn->early_data_state == S2N_EARLY_DATA_REJECTED) /* When early data was rejected */) {
+            POSIX_GUARD(s2n_stuffer_reread(&conn->in));
+            POSIX_GUARD_RESULT(s2n_early_data_record_bytes(conn, s2n_stuffer_data_available(&conn->in)));
+            POSIX_GUARD_RESULT(s2n_wipe_record(conn));
+            return S2N_SUCCESS;
+        }
+        POSIX_GUARD(r);
     }
 
     if (isSSLv2) {
@@ -1068,8 +1165,12 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
     /* Now we have a record, but it could be a partial fragment of a message, or it might
      * contain several messages.
      */
-    S2N_ERROR_IF(record_type == TLS_APPLICATION_DATA, S2N_ERR_BAD_MESSAGE);
-    if (record_type == TLS_CHANGE_CIPHER_SPEC) {
+
+    if (record_type == TLS_APPLICATION_DATA) {
+        POSIX_ENSURE(conn->early_data_expected, S2N_ERR_BAD_MESSAGE);
+        POSIX_GUARD_RESULT(s2n_early_data_validate_recv(conn));
+        POSIX_BAIL(S2N_ERR_EARLY_DATA_BLOCKED);
+    } else if (record_type == TLS_CHANGE_CIPHER_SPEC) {
         /* TLS1.3 can receive unexpected CCS messages at any point in the handshake
          * due to a peer operating in middlebox compatibility mode.
          * However, when operating in QUIC mode, S2N should not accept ANY CCS messages,
@@ -1086,9 +1187,7 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
         POSIX_GUARD(s2n_stuffer_wipe(&conn->handshake.io));
 
         /* We're done with the record, wipe it */
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-        conn->in_status = ENCRYPTED;
+        POSIX_GUARD_RESULT(s2n_wipe_record(conn));
 
         /* Advance the state machine if this was an expected message */
         if (EXPECTED_RECORD_TYPE(conn) == TLS_CHANGE_CIPHER_SPEC && !CONNECTION_IS_WRITER(conn)) {
@@ -1104,10 +1203,8 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
         /* Ignore record types that we don't support */
 
         /* We're done with the record, wipe it */
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-        conn->in_status = ENCRYPTED;
-        return 0;
+        POSIX_GUARD_RESULT(s2n_wipe_record(conn));
+        return S2N_SUCCESS;
     }
 
     /* Record is a handshake message */
@@ -1124,10 +1221,8 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
             /* Break out of this inner loop, but since we're not changing the state, the
              * outer loop in s2n_handshake_io() will read another record.
              */
-            POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-            POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-            conn->in_status = ENCRYPTED;
-            return 0;
+            POSIX_GUARD_RESULT(s2n_wipe_record(conn));
+            return S2N_SUCCESS;
         }
 
         s2n_cert_auth_type client_cert_auth_type;
@@ -1191,11 +1286,8 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
     }
 
     /* We're done with the record, wipe it */
-    POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-    POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-    conn->in_status = ENCRYPTED;
-
-    return 0;
+    POSIX_GUARD_RESULT(s2n_wipe_record(conn));
+    return S2N_SUCCESS;
 }
 
 static int s2n_handle_retry_state(struct s2n_connection *conn)
@@ -1212,6 +1304,9 @@ static int s2n_handle_retry_state(struct s2n_connection *conn)
         S2N_ERROR_PRESERVE_ERRNO();
     }
 
+    /* Resume the handshake */
+    conn->handshake.paused = false;
+
     if (!CONNECTION_IS_WRITER(conn)) {
         /* We're done parsing the record, reset everything */
         POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
@@ -1238,6 +1333,7 @@ static int s2n_handle_retry_state(struct s2n_connection *conn)
     } else {
         /* The read handler processed the record successfully, we are done with this
          * record. Advance the state machine. */
+        POSIX_GUARD(s2n_tls13_handle_secrets(conn));
         POSIX_GUARD(s2n_advance_message(conn));
     }
 
@@ -1249,7 +1345,7 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(blocked);
 
-    while (ACTIVE_STATE(conn).writer != 'B') {
+    while (ACTIVE_STATE(conn).writer != 'B' && ACTIVE_MESSAGE(conn) != conn->handshake.end_of_messages) {
         errno = 0;
         s2n_errno = S2N_ERR_OK;
 
@@ -1287,6 +1383,9 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
 
                 if (s2n_errno == S2N_ERR_ASYNC_BLOCKED) {
                     *blocked = S2N_BLOCKED_ON_APPLICATION_INPUT;
+                    conn->handshake.paused = true;
+                } else if (s2n_errno == S2N_ERR_EARLY_DATA_BLOCKED) {
+                    *blocked = S2N_BLOCKED_ON_EARLY_DATA;
                 }
 
                 S2N_ERROR_PRESERVE_ERRNO();
@@ -1304,6 +1403,9 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
 
                 if (s2n_errno == S2N_ERR_ASYNC_BLOCKED) {
                     *blocked = S2N_BLOCKED_ON_APPLICATION_INPUT;
+                    conn->handshake.paused = true;
+                } else if (s2n_errno == S2N_ERR_EARLY_DATA_BLOCKED) {
+                    *blocked = S2N_BLOCKED_ON_EARLY_DATA;
                 }
 
                 S2N_ERROR_PRESERVE_ERRNO();
