@@ -664,8 +664,8 @@ int s2n_security_policies_init()
         }
 
         if (security_policy != &security_policy_null) {
-            /* catch any offending security policy that does not support P-256 */
-            S2N_ERROR_IF(!s2n_ecc_preferences_includes_curve(ecc_preference, TLS_EC_CURVE_SECP_256_R1), S2N_ERR_INVALID_SECURITY_POLICY);
+            /* All policies must have at least one ecc curve configured. */
+            S2N_ERROR_IF(ecc_preference->count == 0, S2N_ERR_INVALID_SECURITY_POLICY);
         }
 
         for (int j = 0; j < cipher_preference->count; j++) {
