@@ -201,7 +201,7 @@ int s2n_server_hello_recv(struct s2n_connection *conn)
         return 0;
     }
 
-    if (s2n_connection_is_session_resumed(conn)) {
+    if (conn->actual_protocol_version < S2N_TLS13 && s2n_connection_is_session_resumed(conn)) {
         POSIX_GUARD(s2n_prf_key_expansion(conn));
     }
 
